@@ -30,19 +30,22 @@
  * now defined - but only within the scope of http_error_handler.
  */
 
-
-//! file: etc/include/http_error.h
-//!  This file implemented a new and improved error handler that let's
-//!  the administrator implement themed error messages on a per-server
-//!  basis.
-//! cvs_version: $Id: http_error.h,v 1.30.2.10 2005/01/03 17:01:16 kiwi Exp $
+/*
+ **! file: etc/include/http_error.h
+ **!  This file implemented a new and improved error handler that let's
+ **!  the administrator implement themed error messages on a per-server
+ **!  basis.
+ */
 
 
-//! class: http_error_handler
-//!  This is the new HTTP error handler, which tried to make all the error
-//!  messages look constant and nice in a per-server basis.
-//!  A possible fixme is the need to clear the template file out of the
-//!  on a change. I will look at this in the near future.
+/*
+ **! class: http_error_handler
+ **!  This is the new HTTP error handler, which tried to make all the error
+ **!  messages look constant and nice in a per-server basis.
+ **!  A possible fixme is the need to clear the template file out of the
+ **!  on a change. I will look at this in the near future.
+ **! name: http_error_handler - new and pretty HTTP error handler.
+ */
 class http_error_handler {
 
   inherit "caudiumlib";
@@ -173,9 +176,9 @@ class http_error_handler {
            else
            {
               if (id->method != "GET" && id->method != "HEAD" && id->method != "POST")
-                 error_message = "Method (" + html_encode_string (id->method) + ") not recognised.";
+                 error_message = "Method (" + _Roxen.html_encode_string (id->method) + ") not recognised.";
               else
-                 error_message = "Unable to locate the file: " + html_encode_string (id->not_query) + ".<br />\n" +
+                 error_message = "Unable to locate the file: " + _Roxen.html_encode_string (id->not_query) + ".<br />\n" +
                          "The page you are looking for may have moved or been removed.";
            }
         }
@@ -188,7 +191,7 @@ class http_error_handler {
         report_error ("Internal server error in error routine:\n" + describe_backtrace(err) + "\n");
 
         /*
-        data =  http_low_answer( 500, "<h1>Error: The server failed to fulfill your query due to an " +
+        data =  Caudium.HTTP.low_answer( 500, "<h1>Error: The server failed to fulfill your query due to an " +
                         "internal error in the error routine.</h1>" );
                         */
         if (id)
@@ -225,19 +228,19 @@ class http_error_handler {
     if ( args->code ) {
       return sprintf( "%d", the_error->code );
     } else if ( args->name ) {
-      return html_encode_string( the_error->name );
+      return _Roxen.html_encode_string( the_error->name );
     } else if ( args->description ) {
       if ( extra_help[ the_error->code ] ) {
 	return
 	  "<h1>" +
-	  html_encode_string( the_error->name ) +
+	  _Roxen.html_encode_string( the_error->name ) +
 	  "</h1>" +
 	  extra_help[ the_error->code ] +
 	  "<br>";
       } else {
 	return
 	  "<h1>" +
-	  html_encode_string( the_error->name ) +
+	  _Roxen.html_encode_string( the_error->name ) +
 	  "</h1>" +
 	  "There is currently no documentation on this error.<br>";
       }
@@ -246,9 +249,9 @@ class http_error_handler {
     } else if ( args->stamp ) {
       return
 	"Generated by " +
-	html_encode_string( caudium->version() ) +
+	_Roxen.html_encode_string( caudium->version() ) +
 	" at " +
-	html_encode_string( ctime( time() ) );
+	_Roxen.html_encode_string( ctime( time() ) );
     } else if ( args->help ) {
       return
 	"<b>Usage: &lt;error <i>arg</i>&gt;</b>\n" +
@@ -278,24 +281,16 @@ class http_error_handler {
     else 
     {
        /* check if they want old-style 404 */
-#ifdef ENABLE_NEW404
-       if (id->conf->query("Old404") && error_code == 404)
-#else
-       if (error_code == 404)
-#endif /* ENABLE_NEW404 */
-       {
-          return http_low_answer (error_code,
-                          replace (parse_rxml (id->conf->query ("ZNoSuchFile"), id ),
-                                  ({ "$File", "$Me" }),
-                                  ({ html_encode_string (id->not_query), 
-                                   id->conf->query ("MyWorldLocation") })));
-       }
+//       if (id->conf->query("Old404") && error_code == 404)
+//       {
+//          return Caudium.HTTP.low_answer (error_code,
+//                          replace (parse_rxml (id->conf->query ("ZNoSuchFile"), id ),
+//                                  ({ "$File", "$Me" }),
+//                                  ({ _Roxen.html_encode_string (id->not_query), 
+//                                   id->conf->query ("MyWorldLocation") })));
+//       }
 
-#ifdef ENABLE_NEW404
        local_template = get_template (id->conf->query ("ErrorTheme"), id);
-#else
-       local_template = get_template ("", id);
-#endif /* ENABLE_NEW404 */
     }
 
     if (!error_code)
@@ -310,7 +305,7 @@ class http_error_handler {
     if ((int)error_name[0..2] == error_code)
       error_name = error_name[3..];
 
-    string error_page = parse_html (get_template_data (local_template->name, id), ([ "error" : _tag_error ]), ([ ]), ([ "code" : error_code, "name" : error_name, "message" : error_message ]));
+    string error_page = Caudium.parse_html (get_template_data (local_template->name, id), ([ "error" : _tag_error ]), ([ ]), ([ "code" : error_code, "name" : error_name, "message" : error_message ]));
 
     if (id)
       error_page = parse_rxml (error_page, id);
